/************************************************************************
 * MAT 259 - Winter 2020                                                *
 * Data Visualization - Project 3                                       *     
 *                                                                      *
 * Author: Guanyu Chen                                                  *
 * Supervisor: George Legrady                                           *
 * TA: Weidi Zhang                                                      *
 * Supported Data Source: LePtC                                         *
 *                                                                      *
 * Purpose: Investigate Connections between top-10 uploaders'           *
 *          numbers of followers and numbers of videos'views in         *
 *          different classes in Bilibili(Chinese Youtube)              *
 *                                                                      *
 * Usage: 1. Press buttons on the left to control different classes     *
 *                                                                      *
 *        2. Press O to show/hide the ordinate axis                     *
 *                                                                      *
 *        3. Press I to show/hide the verbal information                *
 *                                                                      *
 *        4. Press P to show/hide points                                *
 *                                                                      *
 *        5. Press L to show/hide curves                                *
 *                                                                      *
 *        6. Press Space to remove all graphs                           *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

import java.util.Map;
import peasy.*;
import controlP5.*;
PeasyCam cam;
ControlP5 cp5;
CheckBox checkbox0, checkbox1, checkbox2, checkbox3, checkbox4, checkbox5, checkbox6, checkbox7, checkbox8;

// initial front
PFont f;

// declare the arraylist to fill the arraylist with video objects
ArrayList<Video>videos = new ArrayList<Video>();

// initial color hashmap
HashMap<String, Integer> clr_map = new HashMap<String, Integer>();

// initial parameters
int numRows;
int week = 7;
int min_count, max_count;
float min_width, max_width;
int[] fontSize = {280, 14, 12};
int[] fontColor = {240, 180};

int baseline_Z = -250;

void setup() {

  size(1250, 650, P3D);
  cam = new PeasyCam(this, 2000);
  cam.setMinimumDistance(200);
  cam.setMaximumDistance(4000);


  f = createFont("Arial", 100, true);

  Table table = loadTable("subset_data.csv", "header");
  numRows = table.getRowCount();

  // import all data into class
  for (int i = 0; i < numRows; i++) {
    int aid = table.getInt(i, 0);
    int up_id = table.getInt(i, 3);
    int view = table.getInt(i, 4);
    int num_followers = table.getInt(i, 5);
    int day_of_week = table.getInt(i, 6);
    String video_class = table.getString(i, 7);
    int share = table.getInt(i, 8);
    int danmaku = table.getInt(i, 9);

    videos.add(new Video(aid, up_id, view, num_followers, day_of_week, video_class, share, danmaku));
  }

  gui();
}


void draw() {
  
  // rotate for view
  rotateX(1.2);
  // initial default color hashmap
  create_clr_map();
  background(30);


  hint(DISABLE_DEPTH_TEST);
  cam.beginHUD();
  if (havetext) drawInfo();
  cp5.draw();
  cam.endHUD();
  hint(ENABLE_DEPTH_TEST);
  
  pushMatrix();
  smooth();
  // draw ordinates
  if (Ord) drawWeek();
  drawData();
  popMatrix();

  /// when interact with cp5, keep camera still.
  if (cp5.getWindow().isMouseOver()) {
    cam.setActive(false);
  } else {
    cam.setActive(true);
  }
}


void drawData() {
  
  for (int i = 0; i < 9; i++) {
    if (class_ctrl[i]) {
      draw_fn(i);
    }
  }
}
